@@ -11,11 +11,12 @@ typedef enum {
 	GNOME_PTY_OPEN_PTY_LASTLOGWTMP,
 	GNOME_PTY_OPEN_NO_DB_UPDATE,
 	GNOME_PTY_RESET_TO_DEFAULTS,
-	GNOME_PTY_CLOSE_PTY
+	GNOME_PTY_CLOSE_PTY,
+	GNOME_PTY_SYNCH
 } GnomePtyOps;
 
 void *update_dbs         (int utmp, int wtmp, int lastlog, char *login_name, char *display_name, char *term_name);
 void *write_login_record (char *login_name, char *display_name, char *term_name, int utmp, int wtmp, int lastlog);
-void write_logout_record (void *data, int utmp, int wtmp);
+void write_logout_record (char *login_name, void *data, int utmp, int wtmp);
 
 #endif
