@@ -719,6 +719,11 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 		 GINT_TO_POINTER(FALSE),
 		 GINT_TO_POINTER(TRUE),
 		 NULL, NULL},
+		/* 2004: Bracketed paste mode. */
+		{2004, &terminal->pvt->screen->bracketed_paste_mode, NULL, NULL,
+		 GINT_TO_POINTER(FALSE),
+		 GINT_TO_POINTER(TRUE),
+		 NULL, NULL,},
 	};
 
 	/* Handle the setting. */
@@ -800,10 +805,12 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 		vte_terminal_emit_resize_window(terminal,
 						(set ? 132 : 80) *
 						terminal->char_width +
-						VTE_PAD_WIDTH * 2,
+						terminal->pvt->inner_border.left +
+                                                terminal->pvt->inner_border.right,
 						terminal->row_count *
 						terminal->char_height +
-						VTE_PAD_WIDTH * 2);
+						terminal->pvt->inner_border.top +
+                                                terminal->pvt->inner_border.bottom);
 		/* Request a resize and redraw. */
 		_vte_invalidate_all(terminal);
 		break;
@@ -821,6 +828,12 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 	case 47:
 	case 1047:
 	case 1049:
+		/* Clear the alternate screen if we're switching
+		 * to it, and home the cursor. */
+		if (set) {
+			_vte_terminal_clear_screen (terminal);
+			_vte_terminal_home_cursor (terminal);
+		}
 		/* Reset scrollbars and repaint everything. */
 		terminal->adjustment->value =
 			terminal->pvt->screen->scroll_delta;
@@ -1219,8 +1232,7 @@ vte_sequence_handler_cs (VteTerminal *terminal, GValueArray *params)
 	    screen->scrolling_region.end == rows - 1) {
 		screen->scrolling_restricted = FALSE;
 	}
-	screen->cursor_current.row = screen->insert_delta + start;
-	screen->cursor_current.col = 0;
+	_vte_terminal_home_cursor (terminal);
 }
 
 /* Restrict scrolling and updates to a subset of the visible lines, because
@@ -3051,9 +3063,11 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 						arg2, arg1);
 				vte_terminal_emit_resize_window(terminal,
 								arg2 +
-								VTE_PAD_WIDTH * 2,
+								terminal->pvt->inner_border.left +
+								terminal->pvt->inner_border.right,
 								arg1 +
-								VTE_PAD_WIDTH * 2);
+								terminal->pvt->inner_border.top +
+								terminal->pvt->inner_border.bottom);
 				i += 2;
 			}
 			break;
@@ -3079,9 +3093,11 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 						arg2, arg1);
 				vte_terminal_emit_resize_window(terminal,
 								arg2 * terminal->char_width +
-								VTE_PAD_WIDTH * 2,
+								terminal->pvt->inner_border.left +
+								terminal->pvt->inner_border.right,
 								arg1 * terminal->char_height +
-								VTE_PAD_WIDTH * 2);
+								terminal->pvt->inner_border.top +
+								terminal->pvt->inner_border.bottom);
 				i += 2;
 			}
 			break;
@@ -3119,7 +3135,8 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 					      &width, &height);
 			g_snprintf(buf, sizeof(buf),
 				   _VTE_CAP_CSI "3;%d;%dt",
-				   width + VTE_PAD_WIDTH, height + VTE_PAD_WIDTH);
+				   width + terminal->pvt->inner_border.left,
+                                   height + terminal->pvt->inner_border.top);
 			_vte_debug_print(VTE_DEBUG_PARSE,
 					"Reporting window location"
 					"(%d++,%d++).\n",
@@ -3130,13 +3147,17 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			/* Send window size, in pixels. */
 			g_snprintf(buf, sizeof(buf),
 				   _VTE_CAP_CSI "4;%d;%dt",
-				   widget->allocation.height - 2 * VTE_PAD_WIDTH,
-				   widget->allocation.width - 2 * VTE_PAD_WIDTH);
+				   widget->allocation.height -
+                                       (terminal->pvt->inner_border.top +
+                                        terminal->pvt->inner_border.bottom),
+				   widget->allocation.width -
+                                       (terminal->pvt->inner_border.left +
+                                        terminal->pvt->inner_border.right));
 			_vte_debug_print(VTE_DEBUG_PARSE,
 					"Reporting window size "
 					"(%dx%dn",
-					width - 2 * VTE_PAD_WIDTH,
-					height - 2 * VTE_PAD_WIDTH);
+					width - (terminal->pvt->inner_border.left + terminal->pvt->inner_border.right),
+					height - (terminal->pvt->inner_border.top + terminal->pvt->inner_border.bottom));
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 18:
@@ -3162,21 +3183,29 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 20:
-			/* Report the icon title. */
+			/* Report a static icon title, since the real
+			   icon title should NEVER be reported, as it
+			   creates a security vulnerability.  See
+			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
+			   and CVE-2003-0070. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-				"Reporting icon title.\n");
+				"Reporting fake icon title.\n");
+			/* never use terminal->icon_title here! */
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "L%s" _VTE_CAP_ST,
-				    terminal->icon_title);
+				    _VTE_CAP_OSC "LTerminal" _VTE_CAP_ST);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 21:
-			/* Report the window title. */
+			/* Report a static window title, since the real
+			   window title should NEVER be reported, as it
+			   creates a security vulnerability.  See
+			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
+			   and CVE-2003-0070. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-					"Reporting window title.\n");
+					"Reporting fake window title.\n");
+			/* never use terminal->window_title here! */
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "l%s" _VTE_CAP_ST,
-				    terminal->window_title);
+				    _VTE_CAP_OSC "lTerminal" _VTE_CAP_ST);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		default:
@@ -3188,9 +3217,11 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 				 * rows. */
 				vte_terminal_emit_resize_window(terminal,
 								terminal->column_count * terminal->char_width +
-								VTE_PAD_WIDTH * 2,
+                                                                terminal->pvt->inner_border.left +
+                                                                terminal->pvt->inner_border.right,
 								param * terminal->char_height +
-								VTE_PAD_WIDTH * 2);
+								terminal->pvt->inner_border.top +
+                                                                terminal->pvt->inner_border.bottom);
 			}
 			break;
 		}
